@@ -1,5 +1,4 @@
 #' @name gl.report.bases
-# PRELIMINARIES -- Set parameters --------------
 #' @title Reports summary of base pair frequencies
 #' @family matched reports
 
@@ -17,11 +16,12 @@
 #' @param plot.colors List of two color names for the borders and fill of the
 #'  plots [default c("#2171B5","#6BAED6")].
 #' @param plot.dir Directory to save the plot RDS files [default as specified 
-#' by the global working directory or tempdir()]
-#' @param plot.file Name for the RDS binary file to save (base name only, exclude extension) [default NULL]
+#' by the global working directory or tempdir()].
+#' @param plot.file Name for the RDS binary file to save (base name only, 
+#' exclude extension) [default NULL].
 #' @param verbose Verbosity: 0, silent or fatal errors; 1, begin and end; 2,
 #' progress log; 3, progress and results summary; 5, full report
-#'  [default NULL, unless specified using gl.set.verbosity]
+#'  [default NULL, unless specified using gl.set.verbosity].
 #' @param ... Parameters passed to function \link[ggplot2]{ggsave}, 
 #'  such as width and height, when the ggplot is to be saved.
 
@@ -35,23 +35,28 @@
 
 #' For presence/absence data (SilicoDArT), it is not possible to count
 #' transversions or transitions or transversions/transitions ratio because the
-#'  SNP data are not available, only a single sequence tag per locus.
+#' SNP data are not available, only a single sequence tag per locus. Only base frequencies
+#' are provided.
 
 #'  A color vector can be obtained with gl.select.colors() and then passed to the function
 #'  with the plot.colors parameter.
 
-#' Themes can be obtained from in \itemize{
-#'  \item \url{https://ggplot2.tidyverse.org/reference/ggtheme.html} and \item
-#'  \url{https://yutannihilation.github.io/allYourFigureAreBelongToUs/ggthemes/}
-#'  }
-#'   If a plot.file is given, the ggplot arising from this function is saved as an "RDS" 
+# Themes can be obtained from in \itemize{
+#  \item \url{https://ggplot2.tidyverse.org/reference/ggtheme.html} and \item
+#  \url{https://yutannihilation.github.io/allYourFigureAreBelongToUs/ggthemes/}
+#  }
+
+#' If a plot.file is given, the ggplot arising from this function is saved as an "RDS" 
 #' binary file using saveRDS(); can be reloaded with readRDS(). A file name must be 
 #' specified for the plot to be saved.
 
 #'  If a plot directory (plot.dir) is specified, the ggplot binary is saved to that
 #'  directory; otherwise to the tempdir(). 
 
-#' @author Custodian: Arthur Georges -- Post to
+#' To avoid issues from inadvertent use of this function in an assignment statement,
+#' the function returns the genlight object unaltered.
+
+#' @author Author(s); Arthur Georges. Custodian: Arthur Georges -- Post to
 #' \url{https://groups.google.com/d/forum/dartr}
 
 #' @examples
@@ -209,7 +214,7 @@ gl.report.bases <- function(x,
     } else {
         cat(paste("  Transitions  :", round(ts, 2), "\n"))
         cat(paste("  Transversions:", round(tv, 2), "\n"))
-        cat(paste("  tv/ts ratio:", round(ratio, 4), "\n\n"))
+        cat(paste("  ts/tv ratio:", round(ratio, 4), "\n\n"))
     }
     
     # Plot the results ----------------- 
