@@ -8,18 +8,24 @@
 #' for each population or the observed heterozygosity for each individual in a
 #' genlight object.
 
-#' @param x Name of the genlight object containing the SNP [required].
+#' @param x Name of the genlight object containing the SNP data [required].
 #' @param method Calculate heterozygosity by population (method='pop') or by
 #' individual (method='ind') [default 'pop'].
 #' @param n.invariant An estimate of the number of invariant sequence tags used
 #' to adjust the heterozygosity rate [default 0].
+#' @param subsample.pop Whether subsample populations to estimate observed 
+#' heterozygosity (see Details) [default FALSE].
+#' @param n.limit Minimum number of individuals that should have a population to 
+#' perform subsampling to estimate heterozygosity [default 10].
 #' @param nboots Number of bootstrap replicates to obtain confidence intervals
 #' [default 0].
-#' @param conf The confidence level of the required interval  [default 0.95].
+#' @param boot.method boostraping across individuals ("ind") or across loci
+#'  ("loc") [default "ind"].
+#' @param conf The confidence level of the required interval [default 0.95].
 #' @param CI.type Method to estimate confidence intervals. One of
 #' "norm", "basic", "perc" or "bca" [default "bca"].
 #' @param ncpus Number of processes to be used in parallel operation. If ncpus
-#' > 1 parallel operation is activated,see "Details" section [default 1].
+#' > 1 parallel operation is activated, see "Details" section [default 1].
 #' @param plot.display Specify if plot is to be produced [default TRUE].
 #' @param plot.theme Theme for the plot. See Details for options
 #' [default theme_dartR()].
@@ -28,11 +34,13 @@
 #' [default gl.colors("dis")].
 #' @param plot.colors.ind List of two color names for the borders and fill of
 #' the plot by individual [default gl.colors(2)].
-#' @param error.bar statistic to be plotted as error bar either "SD" (standard 
-#' deviation) or "SE" (standard error) or "CI" (confident intervals)
+#' @param error.bar Statistic to be plotted as error bar either "SD" (standard 
+#' deviation) or "SE" (standard error) or "CI" (confidence intervals)
 #'  [default "SD"].
-#' @param save2tmp If TRUE, saves any ggplots and listings to the session
-#' temporary directory (tempdir) [default FALSE].
+#' @param plot.dir Directory to save the plot RDS files [default as specified 
+#' by the global working directory or tempdir()].
+#' @param plot.file Name for the RDS binary file to save (base name only, 
+#' exclude extension) [default NULL].
 #' @param verbose Verbosity: 0, silent or fatal errors; 1, begin and end; 2,
 #' progress log; 3, progress and results summary; 5, full report
 #' [default NULL, unless specified using gl.set.verbosity].
@@ -95,27 +103,41 @@
 #' }
 
 #'\strong{ Function's output }
-
+#'
 #' Output for method='pop' is an ordered barchart of observed heterozygosity,
-#' unbiased expected heterozygosity and FIS (Inbreeding coefficient) across populations
-#' together with a table of mean observed and expected heterozygosities and FIS
-#' by population and their respective standard deviations (SD).
+#' unbiased expected heterozygosity and FIS (Inbreeding coefficient) across 
+#' populations together with a table of mean observed and expected 
+#' heterozygosities and FIS by population and their respective standard 
+#' deviations (SD).
 
 #' In the output, it is also reported by population: the number of loci used to
-#'  estimate heterozygosity(n.Loc), the number of polymorphic loci (polyLoc),
+#'  estimate heterozygosity (n.Loc), the number of polymorphic loci (polyLoc),
 #'  the number of monomorphic loci (monoLoc) and loci with all missing data
 #'   (all_NALoc).
-
+#'   
 #' Output for method='ind' is a histogram and a boxplot of heterozygosity across
 #' individuals.
-
-#'  Plots and table are saved to the session temporary directory (tempdir)
-
-#'  Examples of other themes that can be used can be consulted in \itemize{
+#' 
+#' If a plot.file is given, the ggplot arising from this function is saved as an 
+#' "RDS" binary file using saveRDS(); can be reloaded with readRDS(). A file 
+#' name must be specified for the plot to be saved.
+#' 
+#' If a plot directory (plot.dir) is specified, the ggplot binary is saved to 
+#' that directory; otherwise to the tempdir(). 
+#'  
+#'  Examples of other themes that can be used can be consulted in: 
+#'  \itemize{
 #'  \item \url{https://ggplot2.tidyverse.org/reference/ggtheme.html} and \item
 #'  \url{https://yutannihilation.github.io/allYourFigureAreBelongToUs/ggthemes/}
 #'  }
 #'  
+#'  \strong{Subsampling populations}
+#'  
+#' To test the effect of five population sample sizes (n = 10, 5, 4, 3, 2) on 
+#' observed heterozygosity estimates, the function subsamples individuals,
+#'  without replacement. The subsampling is repeated 10 times for each sample
+#'   size n. This approach is an implementation of Schmidt et al (2021). 
+#'  
 #'   \strong{Error bars}
 #'  
 #'  The best method for presenting or assessing genetic statistics depends on 
@@ -128,10 +150,10 @@
 #'  
 #'  - Advantage: Confidence intervals give a range in which the true parameter 
 #'  (like a population mean) is likely to fall, given the data and a specified 
-#'  probability (like 95%).
+#'  probability (like 95\%).
 #'  
 #'  - In Context: For genetic statistics, if you're estimating a parameter,
-#'   a 95% CI gives you a range in which you're 95% confident the true parameter
+#'   a 95\% CI gives you a range in which you're 95\% confident the true parameter
 #'    lies.
 #'  
 #'   \strong{2. Standard Deviation ("SD"):}
@@ -170,7 +192,7 @@
 #'   analyze and present their data, depending on their research questions and 
 #'   the nature of the data.
 #'   
-#'  \strong{Confident Intervals}
+#'  \strong{Confidence Intervals}
 #'
 #' The uncertainty of a parameter, in this case the mean of the statistic, can
 #' be summarised by a confidence interval (CI) which includes the true parameter
@@ -213,7 +235,7 @@
 #'      (1997, p. 194) suggest that the number of bootstrap replicates should
 #'      be between 1000 and 2000.
 #'
-#'  \strong{It is important} to note that unreliable confident intervals will be
+#'  \strong{It is important} to note that unreliable confidence intervals will be
 #'   obtained if too few number of bootstrap replicates are used.
 #'   Therefore, the function \link[boot]{boot.ci} will throw warnings and errors
 #'    if bootstrap replicates are too few. Consider increasing the number of
@@ -252,12 +274,18 @@
 #'     Opening and terminating R sessions in each core involves a significant
 #'     amount of processing time, therefore parallelisation in Windows machines
 #'    is only quicker than not using parallelisation when nboots > 1000-2000.
+#'    
 #' @author Custodian: Luis Mijangos (Post to
 #' \url{https://groups.google.com/d/forum/dartr})
 #'
 #' @references
-#' Nei, M. (1978). Estimation of average heterozygosity and genetic distance
-#' from a small number of individuals. Genetics, 89(3), 583-590.
+#' \itemize{
+#' \item Nei, M. (1978). Estimation of average heterozygosity and genetic 
+#' distance from a small number of individuals. Genetics, 89(3), 583-590.
+#' \item Schmidt, T. L., Jasper, M. E., Weeks, A. R., & Hoffmann, A. A. (2021).
+#'  Unbiased population heterozygosity estimates from genome‐wide sequence data.
+#'   Methods in Ecology and Evolution, 12(10), 1888-1898.
+#'   }
 #'
 #' @examples
 #'  \donttest{
@@ -266,6 +294,7 @@
 #' df <- gl.report.heterozygosity(platypus.gl,method='ind')
 #' n.inv <- gl.report.secondaries(platypus.gl)
 #' gl.report.heterozygosity(platypus.gl, n.invariant = n.inv[7, 2])
+#' gl.report.heterozygosity(platypus.gl, subsample.pop = TRUE)
 #' }
 #' df <- gl.report.heterozygosity(platypus.gl)
 
@@ -273,12 +302,15 @@
 
 #' @export
 #' @return A dataframe containing population labels, heterozygosities, FIS,
-#' their standard deviations and sample sizes
+#' their standard deviations and sample sizes.
 
 gl.report.heterozygosity <- function(x,
                                      method = "pop",
                                      n.invariant = 0,
+                                     subsample.pop = FALSE,
+                                     n.limit = 10,
                                      nboots = 0,
+                                     boot.method = "ind",
                                      conf = 0.95,
                                      CI.type = "bca",
                                      ncpus = 1,
@@ -286,80 +318,12 @@ gl.report.heterozygosity <- function(x,
                                      plot.theme = theme_dartR(),
                                      plot.colors.pop = gl.colors("dis"),
                                      plot.colors.ind = gl.colors(2),
+                                     plot.file = NULL,
+                                     plot.dir = NULL,
                                      error.bar = "SD",
-                                     save2tmp = FALSE,
                                      verbose = NULL) {
-  # standard error function
-  std.error <- function(x) {
-    res <- sd(x, na.rm = TRUE) / sqrt(length(x))
-    return(res)
-  }
   
-  # bootstrapping function
-  pop.het <- function(x,
-                      indices,
-                      n.invariant) {
-    pop.het_fun <- function(df,
-                            n.invariant) {
-      Ho.loc <- colMeans(df == 1, na.rm = TRUE)
-      n_loc <- apply(df, 2, function(y) {
-        sum(!is.na(y))
-      })
-      Ho.adj.loc <- Ho.loc * n_loc / (n_loc + n.invariant)
-      q_freq <- colMeans(df, na.rm = TRUE) / 2
-      p_freq <- 1 - q_freq
-      He.loc <- 2 * p_freq * q_freq
-      n_ind <- apply(df, 2, function(y) {
-        sum(!is.na(y))
-      })
-      ### CP ### Unbiased He (i.e. corrected for sample size) hard
-      # coded for diploid
-      uHe.loc <-
-        (2 * as.numeric(n_ind) / (2 * as.numeric(n_ind) - 1)) * He.loc
-      Hexp.adj.loc <- He.loc * n_loc / (n_loc + n.invariant)
-      
-      FIS.loc <- 1 - (Ho.loc / He.loc)
-      
-      all.res <- c(
-        Ho.loc = mean(Ho.loc, na.rm = TRUE),
-        Ho.adj.loc = mean(Ho.adj.loc, na.rm = TRUE),
-        He.loc = mean(He.loc, na.rm = TRUE),
-        uHe.loc = mean(uHe.loc, na.rm = TRUE),
-        Hexp.adj.loc = mean(Hexp.adj.loc, na.rm = TRUE),
-        FIS.loc = mean(FIS.loc, na.rm = TRUE)
-      )
-      
-      return(all.res)
-    }
-    
-    df <- x[, indices]
-    
-    res <- pop.het_fun(df,
-                       n.invariant = n.invariant)
-    
-    return(res)
-    
-  }
-  
-  # Counting individuals function
-  ind.count <- function(x) {
-    # the loci that are completely missing
-    loci.na <-
-      which(colSums(is.na(as.matrix(x))) == nrow(as.matrix(x)))
-    # the number of samples in the matrix the number of non-genotyped
-    # samples remove the loci that are completely missing
-    if (length(loci.na) > 0) {
-      nind <-
-        mean(nrow(as.matrix(x)) -
-               colSums(is.na(as.matrix(x)))[-loci.na])
-      # the number of samples in the matrix the number of
-      # non-genotyped samples
-    } else {
-      nind <- mean(nrow(as.matrix(x)) - colSums(is.na(as.matrix(x))))
-    }
-    
-    return(nind)
-  }
+  # Utils functions are in utils.het.report.r
   
   # setting parallel
   # if (ncpus > 1) {
@@ -374,6 +338,10 @@ gl.report.heterozygosity <- function(x,
   
   # SET VERBOSITY
   verbose <- gl.check.verbosity(verbose)
+  if(verbose==0){plot.display <- FALSE}
+  
+  # SET WORKING DIRECTORY
+  plot.dir <- gl.check.wd(plot.dir,verbose=0)
   
   # FLAG SCRIPT START
   funname <- match.call()[[1]]
@@ -425,7 +393,7 @@ gl.report.heterozygosity <- function(x,
   
   if( nboots == 0 & error.bar == "CI" ){
     cat(error(
-"  Number of boostraps ('nboots' parameter) must be > 0 to calculate confident 
+"  Number of boostraps ('nboots' parameter) must be > 0 to calculate confidence 
    intervals \n"))
     stop()
   }
@@ -452,6 +420,11 @@ gl.report.heterozygosity <- function(x,
       pop(x) <- as.factor(pop(x))
     }
     
+    # Subsampling populations as in Schmidt (2021)
+    if(subsample.pop==TRUE){
+    res_sub <- utils.subsample.pop(x,n.limit = n.limit)
+    }
+    
     # Split the genlight object into a list of populations
     sgl <- seppop(x)
     
@@ -524,7 +497,6 @@ gl.report.heterozygosity <- function(x,
       poly_loc <-  c(poly_loc, nLoc(y_temp))
       mono_loc <- c(mono_loc, (nLoc(hold) - nLoc(y_temp) - length(loc.list_NA)))
       all_na_loc <- c(all_na_loc, length(loc.list_NA))
-      
     }
     
     # Apply correction CP ###
@@ -612,14 +584,52 @@ gl.report.heterozygosity <- function(x,
       
     }
     
-    # bootstrapping
-    # creating matrices to store CI
+    # Prep for results
+    df.base <-
+      data.frame(
+        pop = popNames(x),
+        n.Ind = round(n_ind, 6),
+        n.Loc = n_loc,
+        n.Loc.adj = n_loc / (n_loc + n.invariant),
+        polyLoc = poly_loc ,
+        monoLoc = mono_loc ,
+        all_NALoc = all_na_loc)
+    
+    if (n.invariant == 0) {
+      df.params <- data.frame(
+        Ho = round(as.numeric(Ho),6),
+        HoSD = round(HoSD,6),
+        HoSE = round(HoSE, 6),
+            
+        He = round(Hexp, 6),
+        HeSD = round(HexpSD, 6),
+        HeSE = round(HexpSE,6),
+       
+        uHe = round(uHexp, 6),
+        uHeSD = round(uHexpSD, 6),
+        uHeSE = round(uHexpSE ,6),
+       
+        FIS = round(FIS,6),
+        FISSD = round(FISSD,6),
+        FISSE = round(FISSE , 6)
+    )
+    } else {
+      df.params <- data.frame(
+        Ho.adj = round(as.numeric(Ho.adj),6),
+        Ho.adjSD = round(Ho.adjSD,6),
+        Ho.adjSE = round(Ho.adjSE, 6),
+        
+        He.adj = round(Hexp.adj, 6),
+        He.adjSD = round(Hexp.adjSD, 6),
+        He.adjSE = round(Hexp.adjSE, 6)
+        )
+    }
+    
+    df <- cbind(df.base, df.params)
     npops <- nPop(x)
-    res_CI <- replicate(npops,
-                        as.data.frame(matrix(nrow = 6, ncol = 2)),
-                        simplify = FALSE)
-  
-    if (npops > 1 & nboots > 0) {
+    
+    # bootstrapping
+    if (nboots > 0) {
       pop_boot <- lapply(sgl, function(y) {
         df <- as.data.frame(as.matrix(y))
         
@@ -627,6 +637,8 @@ gl.report.heterozygosity <- function(x,
           data = df,
           statistic = pop.het,
           n.invariant = n.invariant,
+          aHet = n.invariant > 0,
+          boot_method = boot.method,
           R = nboots,
           parallel = parallel,
           ncpus = ncpus
@@ -635,17 +647,29 @@ gl.report.heterozygosity <- function(x,
       })
       
       # confidence intervals
-      pop_res <- rbind(Ho, Ho.adj, Hexp, uHexp,Hexp.adj, FIS)
+      
+      # creating matrices to store CI
+      nparams <- ncol(pop_boot[[1]]$t)
+      res_CI <- replicate(npops,
+                          as.data.frame(matrix(nrow = nparams, ncol = 2)),
+                          simplify = FALSE)
+      
+      if(nparams == 4) {
+        pop_res <- rbind(Ho, Hexp, uHexp, FIS)
+      } else {
+        pop_res <- rbind(Ho.adj, Hexp.adj)
+      }
       
       for (pop_n in 1:length(sgl)) {
-        for (stat_n in 1:6) {
+        for (stat_n in seq_len(nparams)) {
           res_CI_tmp <- boot::boot.ci(
             boot.out = pop_boot[[pop_n]],
             conf = conf,
             type = CI.type,
-            index = stat_n,
-            t0 =  pop_res[stat_n, pop_n],
-            t = pop_boot[[pop_n]]$t[, stat_n]
+            index = stat_n
+            # ,
+            # t0 =  pop_res[stat_n, pop_n],
+            # t = pop_boot[[pop_n]]$t[, stat_n]
           )
           
           res_CI[[pop_n]][stat_n,] <-
@@ -654,170 +678,29 @@ gl.report.heterozygosity <- function(x,
         }
       }
       
-      
-    }
-    
-    if (npops == 1 & nboots > 0) {
-      # observed values
-      pop_res <- rbind(Ho, Ho.adj, Hexp, uHexp,Hexp.adj, FIS)
-      res_CI <- as.data.frame(matrix(nrow = 6, ncol = 2))
-      df <- as.data.frame(as.matrix(sgl[[1]]))
-      
-      # bootstrapping
-      pop_boot <- boot::boot(
-        data = df,
-        statistic = pop.het,
-        R = nboots,
-        n.invariant = n.invariant,
-        parallel = parallel,
-        ncpus = ncpus
-      )
-      
-      # confidence intervals
-      for (stat_n in 1:6) {
-        res_CI_tmp <-
-          boot::boot.ci(
-            boot.out = pop_boot,
-            conf = conf,
-            type = CI.type,
-            index = stat_n,
-            t0 =  pop_res[stat_n],
-            t = pop_boot$t[, stat_n]
-          )
-        
-        res_CI[stat_n,] <- tail(as.vector(res_CI_tmp[[4]]), 2)
-        
-      }
-    }
-    
-     if (npops > 2 & nboots > 0) {
-      names(res_CI) <- popNames(x)
-      
-      CI <- lapply(res_CI, function(y) {
-        y <-  c(y[,1],y[,2])
-        return(y)
+      # Build df with CI
+      params_names <- names(df.params)[grep(pattern = "SD|SE", x = names(df.params), invert = TRUE)]
+      lCI_df <- lapply(seq_len(nparams), function(rn, lres=res_CI, nms=params_names) {
+        df <- do.call(rbind, lapply(lres, "[", rn,))
+        names(df) <- paste0(params_names[rn], c("LCI", "HCI"))
+        return(df)
       })
+      df.CI <- do.call(cbind, lCI_df)
+      df <- cbind(df, df.CI)
       
-      stat_list <- as.list(1:12)
-      for(stat_i in 1:12){
-        stat_list[[stat_i]] <- unlist(lapply(CI,"[",stat_i))
+      # set up name order for df
+      sfx <- c("", "SD", "SE", "LCI", "HCI")
+      nms.order <- vector("character", length = nparams * length(sfx))
+      for(rn in seq_len(nparams)) {
+        nms.order[seq_len(length(sfx)) + length(sfx) * (rn - 1)] <- 
+          paste0(params_names[rn], sfx)
       }
-      
-     }
-    
-    if (npops == 1 & nboots > 0) {
-      CI <- c(res_CI[,1],res_CI[,2])
-      
-      stat_list <- as.list(1:12)
-      for(stat_i in 1:12){
-        stat_list[[stat_i]] <- CI[stat_i]
-      }
-  
+      # cbind CI to existing results
+      df <- df[, c(names(df.base), nms.order)]
     }
     
-    ### CP ###
-    if(nboots > 0){
-    
-    df <-
-      data.frame(
-        pop = popNames(x),
-        n.Ind = round(n_ind, 6),
-        n.Loc = n_loc,
-        n.Loc.adj = n_loc / (n_loc + n.invariant),
-        polyLoc = poly_loc ,
-        monoLoc = mono_loc ,
-        all_NALoc = all_na_loc,
-        
-        Ho = round(as.numeric(Ho),6),
-        HoSD = round(HoSD,6),
-        HoSE = round(HoSE, 6),
-        HoLCI = round(stat_list[[1]], 6),
-        HoHCI = round(stat_list[[7]], 6),
-        
-        Ho.adj = round(as.numeric(Ho.adj),6),
-        Ho.adjSD = round(Ho.adjSD,6),
-        Ho.adjSE = round(Ho.adjSE, 6),
-        Ho.adjLCI = round(stat_list[[2]], 6),
-        Ho.adjHCI = round(stat_list[[8]], 6),
-        
-        He = round(Hexp, 6),
-        HeSD = round(HexpSD, 6),
-        HeSE = round(HexpSE,6),
-        HeLCI = round(stat_list[[3]], 6),
-        HeHCI = round(stat_list[[9]], 6),
-        
-        uHe = round(uHexp, 6),
-        uHeSD = round(uHexpSD, 6),
-        uHeSE = round(uHexpSE ,6),
-        uHeLCI = round(stat_list[[4]], 6),
-        uHeHCI = round(stat_list[[10]], 6),
-        
-        He.adj = round(Hexp.adj, 6),
-        He.adjSD = round(Hexp.adjSD, 6),
-        He.adjSE = round(Hexp.adjSE, 6),
-        He.adjLCI = round(stat_list[[5]], 6),
-        He.adjHCI = round(stat_list[[11]], 6),
-        
-        FIS = round(FIS,6),
-        FISSD = round(FISSD,6),
-        FISSE = round(FISSE , 6),
-        FISLCI = round(stat_list[[6]], 6),
-        FISHCI = round(stat_list[[12]], 6)
-      )
-    }else{
-      
-      df <-
-        data.frame(
-          pop = popNames(x),
-          n.Ind = round(n_ind, 6),
-          n.Loc = n_loc,
-          n.Loc.adj = n_loc / (n_loc + n.invariant),
-          polyLoc = poly_loc ,
-          monoLoc = mono_loc ,
-          all_NALoc = all_na_loc,
-          
-          Ho = round(as.numeric(Ho),6),
-          HoSD = round(HoSD,6),
-          HoSE = round(HoSE, 6),
-          HoLCI = NA,
-          HoHCI = NA,
-          
-          Ho.adj = round(as.numeric(Ho.adj),6),
-          Ho.adjSD = round(Ho.adjSD,6),
-          Ho.adjSE = round(Ho.adjSE, 6),
-          Ho.adjLCI = NA,
-          Ho.adjHCI = NA,
-          
-          He = round(Hexp, 6),
-          HeSD = round(HexpSD, 6),
-          HeSE = round(HexpSE,6),
-          HeLCI = NA,
-          HeHCI = NA,
-          
-          uHe = round(uHexp, 6),
-          uHeSD = round(uHexpSD, 6),
-          uHeSE = round(uHexpSE ,6),
-          uHeLCI = NA,
-          uHeHCI = NA,
-          
-          He.adj = round(Hexp.adj, 6),
-          He.adjSD = round(Hexp.adjSD, 6),
-          He.adjSE = round(Hexp.adjSE, 6),
-          He.adjLCI = NA,
-          He.adjHCI = NA,
-          
-          FIS = round(FIS,6),
-          FISSD = round(FISSD,6),
-          FISSE = round(FISSE , 6),
-          FISLCI = NA,
-          FISHCI = NA
-        )
-    }
-    ##########
-    
     if (plot.display) {
-      
-      error_L <- error_H <- value <- color <- variable <- He.adj <- NULL
+      res.mean <- subsample <- error_L <- error_H <- value <- color <- variable <- He.adj <- res_SE <- NULL
       
       # printing plots and reports assigning colors to populations
       if (is(plot.colors.pop, "function")) {
@@ -919,7 +802,47 @@ gl.report.heterozygosity <- function(x,
                               ymax = error_H), 
                           width=0.5)+
             ggtitle(label = "Heterozygosities and FIS by Population",
-                    subtitle = "Error bars show Confident Intervals")
+                    subtitle = "Error bars show Confidence Intervals")
+        }
+        
+        # Subsampling populations as in Schmidt (2021)
+        if(subsample.pop==TRUE){
+          
+          res_sub_plot <- res_sub
+          res_sub_plot$pop <- as.factor(res_sub_plot$pop)
+          res_sub_plot$subsample <- as.factor(res_sub_plot$subsample )
+          res_sub_plot$color <- rep(colors_pops,each=5)
+          
+          res_sub_plot_2 <- reshape2::melt(res_sub_plot, id = c("pop", "color", "subsample","res_SE"))
+          
+          p4 <- ggplot(res_sub_plot_2,aes(x=subsample,y=value))+
+            
+            geom_bar(stat = "identity", 
+                     color = "black", 
+                     fill = res_sub_plot_2$color,
+                     position = position_dodge())+ 
+            facet_wrap(~variable, nrow=1) +
+            facet_wrap(~pop)+
+            geom_errorbar(aes(ymin = value, 
+                              ymax = value + res_SE), 
+                          width=0.5)+
+            ggtitle(label = "Observed heterozygosity in populations subsamples",
+                    subtitle = "Error bars show Standard Error") +
+            plot.theme +
+            theme(
+              axis.ticks.x = element_blank(),
+              axis.text.x = element_text(
+                face = "bold",
+                size = 12
+              ),
+              axis.title.x = element_blank(),
+              axis.ticks.y = element_blank(),
+              axis.title.y = element_blank(),
+              legend.position = "none"
+            )
+          
+          p3 <-  p3 / p4
+          
         }
         
       } else {
@@ -987,58 +910,41 @@ gl.report.heterozygosity <- function(x,
       cat("\n  No. of loci =", nLoc(x), "\n")
       cat("  No. of individuals =", nInd(x), "\n")
       cat("  No. of populations =", nPop(x), "\n")
-      cat("    Minimum Observed Heterozygosity: ", round(min(df$Ho, na.rm = TRUE), 6))
-      if (n.invariant > 0) {
-        cat("   [Corrected:", round(min(df$Ho.adj, na.rm = TRUE), 6), "]\n")
-      } else {
-        cat("\n")
-      }
-      cat("    Maximum Observed Heterozygosity: ", round(max(df$Ho, na.rm = TRUE), 6))
-      if (n.invariant > 0) {
-        cat("   [Corrected:", round(max(df$Ho.adj, na.rm = TRUE), 6), "]\n")
-      } else {
-        cat("\n")
-      }
-      cat("    Average Observed Heterozygosity: ",
-          round(mean(df$Ho, na.rm = TRUE), 6))
-      if (n.invariant > 0) {
-        cat("   [Corrected:", round(mean(df$Ho.adj, na.rm = TRUE), 6), "]\n\n")
-      } else {
-        cat("\n\n")
-      }
-      cat("    Minimum Unbiased Expected Heterozygosity: ",
-          round(min(df$uHe, na.rm = TRUE), 6))
-      if (n.invariant > 0) {
-        cat("   [Corrected:", round(min(df$He.adj, na.rm = TRUE), 6), "]\n")
-      } else {
-        cat("\n")
-      }
-      cat("    Maximum Unbiased Expected Heterozygosity: ",
-          round(max(df$uHe, na.rm = TRUE), 6))
-      if (n.invariant > 0) {
-        cat("   [Corrected:", round(max(df$He.adj, na.rm = TRUE), 6), "]\n")
-      } else {
-        cat("\n")
-      }
-      cat("    Average Unbiased Expected Heterozygosity: ",
-          round(mean(df$uHe, na.rm = TRUE), 6))
-      if (n.invariant > 0) {
-        cat("   [Corrected:", round(mean(df$He.adj, na.rm = TRUE), 6), "]\n\n")
-      } else {
-        cat("\n\n")
-      }
+      if (n.invariant == 0) {
+      cat("    Minimum Observed Heterozygosity: ", round(min(df$Ho, na.rm = TRUE), 6), "\n")
+      cat("    Maximum Observed Heterozygosity: ", round(max(df$Ho, na.rm = TRUE), 6), "\n")
+      cat("    Average Observed Heterozygosity: ", round(mean(df$Ho, na.rm = TRUE), 6), "\n\n")
       
-      if (n.invariant > 0) {
+      cat("    Minimum Unbiased Expected Heterozygosity: ",
+          round(min(df$uHe, na.rm = TRUE), 6), "\n")
+      cat("    Maximum Unbiased Expected Heterozygosity: ",
+          round(max(df$uHe, na.rm = TRUE), 6), "\n")
+      cat("    Average Unbiased Expected Heterozygosity: ",
+          round(mean(df$uHe, na.rm = TRUE), 6), "\n")
+      cat("  Heterozygosity estimates not corrected for uncalled invariant loci\n")
+      
+      } else {
+        
+        cat("    Minimum Observed adjusted Heterozygosity: ", 
+            round(min(df$Ho.adj, na.rm = TRUE), 6), "\n")
+        cat("    Maximum Observed adjusted Heterozygosity: ", 
+            round(max(df$Ho.adj, na.rm = TRUE), 6), "\n")
+        cat("    Average Observed adjusted Heterozygosity: ",
+            round(mean(df$Ho.adj, na.rm = TRUE), 6), "\n\n")
+        cat("    Minimum Unbiased adjusted Expected Heterozygosity: ", 
+            round(min(df$He.adj, na.rm = TRUE), 6), "\n")
+        cat("    Maximum Unbiased adjusted Expected Heterozygosity: ", 
+            round(max(df$He.adj, na.rm = TRUE), 6), "\n")
+        cat("    Average Unbiased adjusted Expected Heterozygosity: ",
+            round(mean(df$He.adj, na.rm = TRUE), 6), "\n\n")
         cat(
           "  Average correction factor for invariant loci =",
           mean(n_loc / (n_loc + n.invariant), na.rm = TRUE),
           "\n"
         )
-      } else {
-        cat("  Heterozygosity estimates not corrected for uncalled invariant loci\n")
       }
     }
-    
+      
     # PRINTING OUTPUTS
     if (plot.display) {
       suppressWarnings(print(p3))
@@ -1084,6 +990,7 @@ gl.report.heterozygosity <- function(x,
     c.hets <- array(NA, nInd(x))
     c.hom0 <- array(NA, nInd(x))
     c.hom2 <- array(NA, nInd(x))
+    c.nloc <- array(NA, nInd(x))
     for (i in 1:nInd(x)) {
       c.na[i] <- sum(is.na(m[i, ]))
       c.hets[i] <-
@@ -1092,13 +999,14 @@ gl.report.heterozygosity <- function(x,
         sum(m[i, ] == 0, na.rm = TRUE) / (nLoc(x) - c.na[i])
       c.hom2[i] <-
         sum(m[i, ] == 2, na.rm = TRUE) / (nLoc(x) - c.na[i])
+      c.nloc[i] <- (nLoc(x) - c.na[i])
     }
     
     # Join the sample sizes with the heterozygosities
     df <-
-      cbind.data.frame(x@ind.names, c.hets, c.hom0, c.hom2)
+      cbind.data.frame(x@ind.names, c.hets, c.hom0, c.hom2,c.nloc)
     names(df) <-
-      c("ind.name", "Ho", "f.hom.ref", "f.hom.alt")
+      c("ind.name", "Ho", "f.hom.ref", "f.hom.alt","n.Loc")
     
     # Boxplot
     if (plot.display) {
@@ -1162,39 +1070,20 @@ gl.report.heterozygosity <- function(x,
       print(p3)
     }
     if (verbose >= 2) {
+      if(subsample.pop==TRUE){
+        print(res_sub, row.names = FALSE)
+      }
       print(df, row.names = FALSE)
     }
   }
   
-  # SAVE INTERMEDIATES TO TEMPDIR
-  if (save2tmp) {
-    # creating temp file names
-    if (plot.display) {
-      temp_plot <- tempfile(pattern = "Plot_")
-    }
-    temp_table <- tempfile(pattern = "Table_")
-    match_call <-
-      paste0(names(match.call()),
-             "_",
-             as.character(match.call()),
-             collapse = "_")
-    # saving to tempdir
-    if (plot.display) {
-      saveRDS(list(match_call, p3), file = temp_plot)
-      if (verbose >= 2) {
-        cat(report("  Saving the ggplot to session tempfile\n"))
-      }
-    }
-    saveRDS(list(match_call, df), file = temp_table)
-    if (verbose >= 2) {
-      cat(report("  Saving tabulation to session tempfile\n"))
-      cat(
-        report(
-          "  NOTE: Retrieve output files from tempdir using
-                    gl.list.reports() and gl.print.reports()\n"
-        )
-      )
-    }
+  # Optionally save the plot ---------------------
+  
+  if(!is.null(plot.file)){
+    tmp <- utils.plot.save(p3,
+                           dir=plot.dir,
+                           file=plot.file,
+                           verbose=verbose)
   }
   
   if (verbose >= 3) {
@@ -1207,6 +1096,10 @@ gl.report.heterozygosity <- function(x,
   }
   
   # RETURN
+  if(subsample.pop==TRUE){
+   return(invisible(list(res_sub,df)))
+  }else{
+  return(invisible(df))
+  }
   
-  invisible(df)
 }
