@@ -13,13 +13,13 @@
 
 #' @param x Name of the genlight object [required].
 #' @param ind.labels If TRUE, individual IDs are shown [default FALSE].
-# @param group.pop If ind.labels is TRUE, group by population [default TRUE].
+#' @param group.pop If ind.labels is TRUE, group by population [default TRUE].
 #' @param label.size Size of the individual labels [default 10].
 #' @param het.only If TRUE, show only the heterozygous state [default FALSE]
 #' @param plot.display If TRUE, the plot is displayed in the plot window
 #' [default TRUE].
 #' @param plot.theme Theme for the plot. See Details for options
-#' [default theme_dartR()].
+#' [default NULL].
 #' @param plot.colors List of four color names for the column fill for homozygous reference,
 #' heterozygous, homozygous alternate, and missing value (NA) [default c("#0000FF","#00FFFF","#FF0000","#e0e0e0")].
 #' @param plot.dir Directory to save the plot RDS files [default as specified 
@@ -58,8 +58,8 @@ gl.smearplot <- function(x,
                         plot.display=TRUE,
                         ind.labels = FALSE,
                         label.size = 10,
-                        #group.pop = FALSE, 
-                        plot.theme = theme_dartR(),
+                        group.pop = FALSE, 
+                        plot.theme = NULL,
                         plot.colors = NULL,
                         plot.file=NULL,
                         plot.dir=NULL,
@@ -189,11 +189,7 @@ gl.smearplot <- function(x,
             name = "Genotype",
             labels = labels_genotype
             # ) + theme_dartR() + theme(
-          ) + theme(
-            legend.position = legend,
-            #axis.text.y = element_text(size = label.size)
-            axis.text.y = element_text(size = label.size)
-          ) +
+          ) + 
           scale_x_discrete(
             breaks = loc_labels,
             labels = as.character(loc_labels),
@@ -201,6 +197,17 @@ gl.smearplot <- function(x,
             position="bottom"
           ) +
         ylab("Individuals")
+        
+        if(!is.null(plot.theme)){
+          p3 <- p3 + plot.theme
+        }
+        
+        p3 <- p3  + theme(
+          legend.position = legend,
+          #axis.text.y = element_text(size = label.size)
+          axis.text.y = element_text(size = label.size)
+        ) 
+        
       } else {
         p3 <-
             ggplot(df.listing, aes(
@@ -214,11 +221,7 @@ gl.smearplot <- function(x,
                 name = "Genotype",
                 labels = labels_genotype
            # ) + theme_dartR() + theme(
-            ) + theme(
-                legend.position = legend,
-                #axis.text.y = element_text(size = label.size)
-                axis.text.y = element_text(size = label.size)
-            ) +
+            ) + 
             scale_x_discrete(
                 breaks = loc_labels,
                 labels = as.character(loc_labels),
@@ -232,6 +235,16 @@ gl.smearplot <- function(x,
                 position="left"
           )
         #ylab("Individuals")
+        
+        if(!is.null(plot.theme)){
+          p3 <- p3 + plot.theme
+        }
+        
+        p3 <- p3  + theme(
+          legend.position = legend,
+          #axis.text.y = element_text(size = label.size)
+          axis.text.y = element_text(size = label.size)
+        ) 
       }
     }
     
@@ -260,16 +273,28 @@ gl.smearplot <- function(x,
             na.value = plot.colors[4],
             name = "Sequence Tag",
             labels = labels_silicodart
-          ) + theme_dartR() + theme(
-            legend.position = legend,
-            axis.text.y = element_text(size = label.size)
-          ) +
+          ) + 
+          # theme_dartR() + theme(
+          #   legend.position = legend,
+          #   axis.text.y = element_text(size = label.size)
+          # ) +
           scale_x_discrete(
             breaks = loc_labels,
             labels = as.character(loc_labels),
             name = "Loci"
           ) +
           ylab("Individuals")
+        
+        if(!is.null(plot.theme)){
+          p3 <- p3 + plot.theme
+        }
+        
+        p3 <- p3  + theme(
+          legend.position = legend,
+          #axis.text.y = element_text(size = label.size)
+          axis.text.y = element_text(size = label.size)
+        ) 
+        
       } else {
         p3 <-
           ggplot(df.listing, aes(
@@ -281,10 +306,11 @@ gl.smearplot <- function(x,
             na.value = plot.colors[4],
             name = "Sequence Tag",
             labels = labels_silicodart
-          ) + theme_dartR() + theme(
-            legend.position = legend,
-            axis.text.y = element_text(size = label.size)
-          ) +
+          ) + 
+          # theme_dartR() + theme(
+          #   legend.position = legend,
+          #   axis.text.y = element_text(size = label.size)
+          # ) +
           scale_x_discrete(
             breaks = loc_labels,
             labels = as.character(loc_labels),
@@ -297,16 +323,27 @@ gl.smearplot <- function(x,
             position="left"
           )
         #ylab("Individuals")
+        
+        if(!is.null(plot.theme)){
+          p3 <- p3 + plot.theme
+        }
+        
+        p3 <- p3  + theme(
+          legend.position = legend,
+          #axis.text.y = element_text(size = label.size)
+          axis.text.y = element_text(size = label.size)
+        ) 
+        
       }
     }
     
     
-    # if (ind.labels==TRUE & group.pop == TRUE) {
-    #     p3 <- p3 + facet_wrap(~ pop,
-    #                           ncol = 1,
-    #                           dir = "v",
-    #                           scales = "free_y")
-    # }
+    if (group.pop == TRUE) {
+        p3 <- p3 + facet_wrap(~ pop,
+                              ncol = 1,
+                              dir = "v",
+                              scales = "free_y")
+    }
     
     # PRINTING OUTPUTS
     print(p3)
